package ovh.herisson.thevoidroad;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.UUID;

import org.bukkit.Bukkit;
import org.jetbrains.annotations.Nullable;

public class DatabaseManager {

	private static DatabaseManager instance;

	public static DatabaseManager getInstance(){
		if (instance == null){
			try {
			   	instance = new DatabaseManager();
			} catch(SQLException e) {
			   	Bukkit.getLogger().warning("Could not Initiate database : " + e);
		   	}
		}
		return instance;
	}

	private Connection con;

	private DatabaseManager() throws SQLException{
		new File("plugins/TheVoidRoad").mkdirs();
		con = DriverManager.getConnection("jdbc:sqlite:plugins/TheVoidRoad/voids.db");
		InitVoids();
		InitConfig();
		InitDeaths();
	}

	// Voids table and methods
	private void InitVoids() throws SQLException{
		String query = "CREATE TABLE IF NOT EXISTS voids (\n"
			+ "id integer PRIMARY KEY,\n"
			+ "uuid text NOT NULL UNIQUE,\n"
			+ "amount INTEGER DEFAULT 0)";

		Statement st = con.createStatement();
		st.execute(query);
		st.close();
	}

	public int getVoids(UUID uuid){
		try {
			Statement st = con.createStatement();
			String query = "SELECT amount FROM voids WHERE uuid = ? LIMIT 1";
			PreparedStatement p = con.prepareStatement(query);
			p.setString(1, uuid.toString());
			ResultSet rs = p.executeQuery();
			int value = rs.getInt(1);
			st.close();
			return value;
		} catch(Exception e){
			e.printStackTrace();
		}
		return 0;
	}

	public void setVoids(UUID uuid, int amount) {
		try {
			Statement st = con.createStatement();
			String query = "INSERT OR REPLACE INTO voids (uuid, amount) VALUES (?, ?)";
			PreparedStatement p = con.prepareStatement(query);
			p.setString(1, uuid.toString());
			p.setInt(2, amount);
			p.execute();
			st.close();
		} catch(Exception e){
			e.printStackTrace();
		}
	}

    public HashMap<UUID, Integer> getVoidsScoreboard() throws SQLException{
		HashMap<UUID, Integer> ret = new HashMap<>();

		Statement st = con.createStatement();
		String query = "SELECT uuid, amount FROM voids";
		Statement p = con.createStatement();
		ResultSet rs = p.executeQuery(query);
		do{
			ret.put(UUID.fromString(rs.getString(1)), rs.getInt(2));
		}while(rs.next());

		st.close();
		return ret;
    }

	// Config table and methods
	private void InitConfig() throws SQLException{
		String query = "CREATE TABLE IF NOT EXISTS config (\n"
			+ "id integer PRIMARY KEY,\n"
			+ "name text NOT NULL UNIQUE,\n"
			+ "value text default NULL)";

		Statement st = con.createStatement();
		st.execute(query);
		st.close();
	}

	public String getConfig(String name){
		try {
			Statement st = con.createStatement();
			String query = "SELECT value FROM config WHERE name = ? LIMIT 1";
			PreparedStatement p = con.prepareStatement(query);
			p.setString(1, name);
			ResultSet rs = p.executeQuery();
			String value = rs.getString(1);
			st.close();
			return value;
		} catch(Exception e){
			e.printStackTrace();
		}
		return null;
	}

	public void setConfig(String name, @Nullable String value){
		try {
			Statement st = con.createStatement();
			String query = "INSERT OR REPLACE INTO config (name, value) VALUES (?, ?)";
			PreparedStatement p = con.prepareStatement(query);
			p.setString(1, name);
			p.setString(2, value);
			p.execute();
			st.close();
		} catch(Exception e){
			e.printStackTrace();
		}
	}

	private void InitDeaths() throws SQLException{
		String query = " CREATE TABLE IF NOT EXISTS deaths (\n"
			+ "id integer PRIMARY KEY,\n"
			+ "uuid text NOT NULL UNIQUE,\n"
			+ "lastDeath INTEGER DEFAULT (unixepoch() / (60 * 60)))";
		Statement st = con.createStatement();
		st.execute(query);
		st.close();
	}

	public void setDeath(UUID uuid) {
		try {
			Statement st = con.createStatement();
			String query = "INSERT OR REPLACE INTO deaths (uuid) VALUES (?)";
			PreparedStatement p = con.prepareStatement(query);
			p.setString(1, uuid.toString());
			p.execute();
			st.close();
		} catch(Exception e){
			e.printStackTrace();
		}
	}

	public boolean hasDiedInHour(UUID uuid){
		try {
			Statement st = con.createStatement();
			String query = "SELECT (unixepoch() / (60 * 60)) - lastDeath <= 0 FROM deaths WHERE uuid = ? LIMIT 1 ;";
			PreparedStatement p = con.prepareStatement(query);
			p.setString(1, uuid.toString());
			ResultSet rs = p.executeQuery();
			boolean value = rs.getBoolean(1);
			st.close();
			return value;
		} catch(Exception e){
			e.printStackTrace();
		}
		return false;
	}

    public void unban(UUID uuid) {
		try {
			Statement st = con.createStatement();
			String query = "INSERT OR REPLACE INTO deaths (uuid, lastDeath) SELECT uuid, lastDeath - 1 FROM deaths WHERE uuid = ?";
			PreparedStatement p = con.prepareStatement(query);
			p.setString(1, uuid.toString());
			p.execute();
			st.close();
		} catch(Exception e){
			e.printStackTrace();
		}
    }

    public void close(){
		try {
			con.close();
		} catch(Exception e){
			e.printStackTrace();
		}
    }

}
